@@ -5,8 +5,8 @@
  * This library was written at the Laboratory of DNA Information Analysis,
  * Human Genome Center, Institute of Medical Science, University of Tokyo,
  * 4-6-1 Shirokanedai, Minato-ku, Tokyo 108-8639, Japan.
- * Contact: mdehoon 'AT' gsc.riken.jp
- * 
+ * Contact: michiel.dehoon 'AT' riken.jp
+ *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation with or without modifications and for any purpose and
  * without fee is hereby granted, provided that any copyright notices
@@ -15,7 +15,7 @@
  * names of the contributors or copyright holders not be used in
  * advertising or publicity pertaining to distribution of the software
  * without specific prior permission.
- * 
+ *
  * THE CONTRIBUTORS AND COPYRIGHT HOLDERS OF THIS SOFTWARE DISCLAIM ALL
  * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE
@@ -24,7 +24,7 @@
  * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
  * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
  * OR PERFORMANCE OF THIS SOFTWARE.
- * 
+ *
  */
 
 #ifndef min
@@ -38,14 +38,14 @@
 #  include <windows.h>
 #endif
 
-#define CLUSTERVERSION "1.50"
+#define CLUSTERVERSION "1.56"
 
 /* Chapter 2 */
-double clusterdistance (int nrows, int ncolumns, double** data, int** mask,
+double clusterdistance(int nrows, int ncolumns, double** data, int** mask,
   double weight[], int n1, int n2, int index1[], int index2[], char dist,
   char method, int transpose);
-double** distancematrix (int ngenes, int ndata, double** data,
-  int** mask, double* weight, char dist, int transpose);
+void distancematrix(int ngenes, int ndata, double** data, int** mask,
+  double* weight, char dist, int transpose, double** distances);
 
 /* Chapter 3 */
 int getclustercentroids(int nclusters, int nrows, int ncolumns,
@@ -53,10 +53,10 @@ int getclustercentroids(int nclusters, int nrows, int ncolumns,
   int transpose, char method);
 void getclustermedoids(int nclusters, int nelements, double** distance,
   int clusterid[], int centroids[], double errors[]);
-void kcluster (int nclusters, int ngenes, int ndata, double** data,
+void kcluster(int nclusters, int ngenes, int ndata, double** data,
   int** mask, double weight[], int transpose, int npass, char method, char dist,
   int clusterid[], double* error, int* ifound);
-void kmedoids (int nclusters, int nelements, double** distance,
+void kmedoids(int nclusters, int nelements, double** distance,
   int npass, int clusterid[], double* error, int* ifound);
 
 /* Chapter 4 */
@@ -71,12 +71,13 @@ typedef struct {int left; int right; double distance;} Node;
  * between the two subnodes that were joined.
  */
 
-Node* treecluster (int nrows, int ncolumns, double** data, int** mask,
+Node* treecluster(int nrows, int ncolumns, double** data, int** mask,
   double weight[], int transpose, char dist, char method, double** distmatrix);
-void cuttree (int nelements, Node* tree, int nclusters, int clusterid[]);
+int sorttree(const int nnodes, Node* tree, const double order[], int indices[]);
+int cuttree(int nelements, const Node* tree, int nclusters, int clusterid[]);
 
 /* Chapter 5 */
-void somcluster (int nrows, int ncolumns, double** data, int** mask,
+void somcluster(int nrows, int ncolumns, double** data, int** mask,
   const double weight[], int transpose, int nxnodes, int nynodes,
   double inittau, int niter, char dist, double*** celldata,
   int clusterid[][2]);
